/* Extensible System Monitor 0.01 

   main.c - Parses the command line options. Starts the main loop.

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h>

#include "main.h"
#include "loop.h"
#include "options.h"
#include "configload.h"
#include "getopt.h"
#include "version.h"

void printhelp();
void printversion();

int main(int argc,char *argv[]){
  pid_t tmppid;
  int c;
  FILE *pidfile;

  /* Copy defaultconfigdir to configdir. */
  configdir = malloc(strlen(defaultconfigdir) + 1);
  configdir[0] = '\000';
  configdir = strcpy(configdir,defaultconfigdir);
  configdir = addslash(configdir);

  /* Parse the command line options. */
  while (1)
    {
      int option_index = 0;
      static struct option long_options[] =
      {
	{"file", 1, 0, 0},
	{"forground", 0, 0, 0},
	{"help", 0, 0, 0},
	{"version", 0, 0, 0},
	{0, 0, 0, 0}
      };
      
      c = getopt_long (argc, argv, "f:hv",
		       long_options, &option_index);
      if (c == -1)
	break;

      switch (c)
	{
	case 0:
	  if (option_index == 0){
	    free(configdir);
	    configdir = malloc(strlen(optarg));
	    strcpy(configdir,optarg);
	  }
	  else if (option_index == 1){
	    daemon_mode = 0;
	  }
	  else if (option_index == 2){
	    printhelp();
	    exit(EXIT_SUCCESS);
	  }
	  else if (option_index == 3){
	    printversion();
	    exit(EXIT_SUCCESS);
	  }
	  break;
	  
	case 'f':
	  free(configdir);
	  configdir = malloc(strlen(optarg));
	  strcpy(configdir,optarg);
	  break;
	  
	case 'h':
	  printhelp();
   	  exit(EXIT_SUCCESS);
	  break;
       
	case 'v':
	  printversion();
	  exit(EXIT_SUCCESS);
	  break;

	case '?':
                 break;

	default:
                 fprintf (stderr,"?? getopt returned character code 0%o ??\n", c);

	}
    }

  if (optind < argc)
    {
      fprintf (stderr,"non-option ARGV-elements: ");
      while (optind < argc)
	fprintf (stderr,"%s ", argv[optind++]);
      fprintf (stderr,"\n");
    }

  /* Fork and become a daemon. */
  if (daemon_mode){
    tmppid = fork();
    if (tmppid > 0){
      exit(EXIT_SUCCESS);
    }
    else if (tmppid < 0){
      #if HAVE_STRERROR
      fprintf(stderr,"Fork failed. - %s\n",strerror(errno));
      #else
      fprintf(stderr,"Fork failed.\n");
      #endif
      exit(EXIT_FAILURE);
    }
  }   

  /* Write our pid to /var/run/esmd.pid */
  pidfile = fopen("/var/run/esmd.pid","w");
  if (pidfile){
    fprintf(pidfile,"%d\n",(int)getpid());
    fclose(pidfile);
  }

  mainloop();

  return 0;
}

void printhelp(){
 printf("Usage: esmd [OPTION]...\n");
 printf("Extensible System Monitor (ESM) daemon.\n");
 printf("\n");
 printf("  -f DIR, --file DIR       Uses DIR as the configuration directory.\n");
 printf("  --forground              Don't run as a daemon.\n");
 printf("  -h, --help               Displays this screen.\n");
 printf("  -v, --version            Displays version information.\n");
 printf("\n");
 printf("Report bugs to pc_toilet@hotmail.com\n");
}

void printversion(){
  printf("Extensible System Monitor Daemon (ESMD) %s\n",version);
  printf("Copyright (C) 1999 Peter Todd\n");
  printf("\n");
  printf("esmd comes with NO WARRANTY,\n");
  printf("to the extent permitted by law.\n");
  printf("You may redistribute copies of esmd\n");
  printf("under the terms of the GNU General Public License.\n");
  printf("For more information about these matters,\n");
  printf("see the files named COPYING.\n");
}

static char *id="@(#) $Id: main.c,v 4.0 1999/10/07 22:56:13 pete Exp $";

/* 
 * $Log: main.c,v $
 * Revision 4.0  1999/10/07 22:56:13  pete
 * Started work on version 1.0
 *
 * Revision 3.2  1999/09/19 18:25:56  pete
 * Added support for automake
 *
 * Revision 3.1  1999/09/19 16:11:29  pete
 * Now supports the HAVE_STRERROR #define
 *
 * Revision 3.0  1999/09/19 01:15:23  pete
 * Started work on 0.03
 *
 * Revision 2.1  1999/09/17 14:37:30  pete
 * Fixed one buffer overrun. I forgot that strlen doesn't include the null
 * terminator while copying defaultconfigdir to configdir.
 *
 * Revision 2.0  1999/09/15 16:05:51  pete
 * Started work on version 0.02
 *
 * Revision 1.14  1999/09/12 12:10:12  pete
 * Added code to write our pid to /var/run/esmd.pid
 *
 * Revision 1.13  1999/09/12 01:20:24  pete
 * Changed daemon to daemon_mode
 *
 * Revision 1.12  1999/09/11 22:17:37  pete
 * Added help and version args.
 *
 * Revision 1.11  1999/09/11 21:46:23  pete
 * Added command-line options:
 *
 * -f --file <file>
 *
 * The config dir path. Defaults to /etc/esm
 *
 * --forground
 *
 * Don't run as a daemon.
 *
 * Revision 1.10  1999/09/11 21:09:51  pete
 * Added #include <getopt.h>
 *
 * Revision 1.9  1999/09/11 21:00:07  pete
 * Honors the daemon option.
 *
 * Revision 1.8  1999/09/11 19:02:57  pete
 * Bug fix.
 *
 * Revision 1.7  1999/09/11 19:02:28  pete
 * Made main() call addslash(configdir)
 *
 * Revision 1.6  1999/09/11 18:56:42  pete
 * Added code to copy defaultconfigdir to configdir.
 *
 * Added return 0; at the end of main() to get rid of a warning.
 *
 * Revision 1.5  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
