/* Msglist function. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "main.h"
#include "msg.h"

/* The syslogmsg list. Each message recived from syslog is put in this list.
   Whenever we are told to print the syslog this list is read out and printed.
*/
struct syslogmsg *syslogmsglist = NULL;
struct syslogmsg *syslogmsglist_last = NULL;

/* The number of messages in the msg list. */
int nummsgs = 0;

/* The highest urgency level of any message recived since the last report was
   sent.
*/
int maxurgency = 0;

void addmsg(struct syslogmsg *newmsg){
  int cursavepri;
  struct syslogmsg *curmsg;

  nummsgs++;

  newmsg->next = NULL;
  newmsg->prev = NULL;
  if (syslogmsglist)
    newmsg->next = syslogmsglist;
  else 
    syslogmsglist_last = newmsg;

  syslogmsglist = newmsg;

  if (syslogmsglist->next)
    syslogmsglist->next->prev = syslogmsglist;

  /* Delete a msg. If no msg has a urgency level low enough to delete it 
     keep retrying with higher and higher savepri's.
  */
  cursavepri = savepri;
  while (nummsgs > maxmsgs){
    curmsg = syslogmsglist_last;
    while (1){
      if (curmsg->urgency < cursavepri){
	delmsg(curmsg);
	break;
      }
      else if (curmsg->prev)
	curmsg = curmsg->prev;
      else
	break;
    }
    cursavepri++;
  }
}

/* Deletes a msg. */
void delmsg(struct syslogmsg *msg){
  nummsgs--;

  /* Is the msg at the start the syslogmsg list? */
  if (msg == syslogmsglist){
    /* Set syslogmsglist to msg->next or otherwise set syslogmsglist and syslogmsglist_last to NULL*/
    if (msg->next)
      syslogmsglist = msg->next;
    else{
      syslogmsglist = syslogmsglist_last = NULL;
    }
  }

  /* Is the msg at the end the syslogmsg list? */
  else if (msg == syslogmsglist_last){
    /* Set syslogmsglist to msg->prev If there is no msg->prev then something was screwed up. If there
       was 1 entry in the syslogmsg list we would have found out before.
     */
    if (msg->prev){
      syslogmsglist_last = msg->prev;
      syslogmsglist_last->next = NULL;
    }
    else{
      fprintf(stderr,"Can't happen\n");
      exit(EXIT_FAILURE);
    }
  }
  
  /* We are in the middle of the syslogmsg list. Fix up the prev and next msgs. */
  else{
    if (!msg->next || !msg->prev){
      fprintf(stderr,"Can't happen\n");
      exit(EXIT_FAILURE);
    }      
    msg->next->prev = msg->prev;
    msg->prev->next = msg->next;
  }

  /* Delete the msg. */
  free(msg->msg);
  free(msg);
}

static char *id="@(#) $Id: msg.c,v 2.0 1999/10/07 23:26:50 pete Exp $";

/* 
 * $Log: msg.c,v $
 * Revision 2.0  1999/10/07 23:26:50  pete
 * Started work on version 1.1
 *
 * Revision 1.4  1999/09/21 16:14:35  pete
 * Culling seems to work right now.
 *
 * Revision 1.3  1999/09/21 02:09:21  pete
 * Fixed the SEGFAULT. But now I get Can't happen. :(
 *
 * Revision 1.2  1999/09/21 02:04:48  pete
 * Almost working. The culling is nice and fast. However I'm getting
 * segfaults when I STAT the list after a cull.
 *
 * Revision 1.1  1999/09/21 01:17:05  pete
 * First version.
 *
 */
